/*
 * $Id$
 */

/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License). You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the License at
 * https://javaserverfaces.dev.java.net/CDDL.html or
 * legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permission and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * [Name of File] [ver.__] [Date]
 *
 * Copyright 2007 Sun Microsystems Inc. All Rights Reserved
 */

package com.sun.faces.util;

import java.util.logging.Level;
import java.text.MessageFormat;

/**
 * <p>
 *   A simple wrapper for java.util.logging.Logger.  This reduces
 *   the number of Logger.isLoggable() calls within the code, and
 *   provides a clear way to create Loggers via the <code>FacesLogger.Logger</code>
 *   enum.
 * </p>
 */
public class FacesLogger {

    /**
     * <p>
     *   An <code>enum</code> of all application <code>Logger</code>s.
     * </p>
     */
    public enum Logger {

        APPLICATION("application"),
        CONFIG("config"),
        CONTEXT("context"),
        LIFECYCLE("lifecycle"),
        MANAGEDBEAN("managedbean"),
        RENDERKIT("renderkit"),
        TAGLIB("taglib"),
        TIMING("timing");

        private static final String LOGGER_RESOURCES
             = "com.sun.faces.LogStrings";
        private static final String FACES_LOGGER_NAME_PREFIX
             = "javax.enterprise.resource.webcontainer.jsf.";
        private String loggerName;


        Logger(String loggerName) {
            this.loggerName = FACES_LOGGER_NAME_PREFIX + loggerName;
        }


        public String getLoggerName() {
            return loggerName;
        }


        public String getResourcesName() {
            return LOGGER_RESOURCES;
        }
        
    }

    private java.util.logging.Logger logger;


    // ------------------------------------------------------------ Constructors


    private FacesLogger(FacesLogger.Logger logger) {

        this.logger = java.util.logging.Logger.getLogger(logger.getLoggerName(),
                                                         logger.getResourcesName());

    }


    // ---------------------------------------------------------- Public Methods


    public static FacesLogger getLogger(FacesLogger.Logger logger) {

        return new FacesLogger(logger);
        
    }


    /**
     * @see java.util.logging.Logger#log(java.util.logging.Level, String)
     */
    public void log(Level level, String message) {

        if (logger.isLoggable(level)) {
            logger.log(level, message);
        }
        
    }


    /**
     * @see java.util.logging.Logger#log(java.util.logging.Level, String, Object[])
     */
    public void log(Level level, String message, Object... params) {

        if (logger.isLoggable(level)) {
            logger.log(level, message, params);
        }

    }


    /**
     * @see java.util.logging.Logger#log(Level, String, Throwable)
     */
    public void log(Level level, String message, Throwable t) {

        if (logger.isLoggable(level)) {
            logger.log(level, message, t);
        }

    }


    /**
     * <p>This calls through to {@link java.util.logging.Logger#log(java.util.logging.Level, String)}
     * passing <code>message</code> and <code>params</code> through
     * {@link java.text.MessageFormat#format(String, Object[])}.<p>
     *
     * <p>This is mostly useful for FINE{ER,EST} logging since the resource
     * bundle isn't used for these calls.</p>
     *
     * @param level the logging Level
     * @param message the message
     * @param params the params, if any
     */
    public void logFormatted(Level level, String message, Object... params) {

        if (logger.isLoggable(level)) {
            if (params != null && params.length > 0) {
                logger.log(level, MessageFormat.format(message, params));
            } else {
                logger.log(level, message);
            }
        }

    }


    /**
     * Used by Timer in order to determine whether an
     * Timer instance will be used or not.
     * @param level
     * @return
     */
    boolean isLoggable(Level level) {
        return logger.isLoggable(level);
    }


}
